import os

import Ice
from flask import Flask
from flask_ldap3_login import AuthenticationResponseStatus
from flask_security import verify_password

from ..extensions import ldap_manager

# Load up Murmur slice file into Ice
Ice.loadSlice(
    "",
    [
        "-I" + Ice.getSliceDir(),
        os.path.join(os.path.dirname(os.path.realpath(__file__)), "Murmur.ice"),
    ],
)
import Murmur


class MumbleAuthenticator(Murmur.ServerUpdatingAuthenticator):
    texture_cache = {}

    def __init__(self, app, logger, ice):
        Murmur.ServerUpdatingAuthenticator.__init__(self)
        self.app: Flask = app
        self.logger = logger
        self.ice = ice

    def authenticate(self, username, password, certlist, certhash, strong, current=None):
        """
        This function is called to authenticate a user
        """
        if username == "SuperUser":
            return (-2, None, None)

        self.logger.info("Mumble auth request for {}".format(username))

        with self.app.app_context():
            user = self.app.security.datastore.find_user(username=username)
            if not user:
                self.logger.warning("Mumble auth: User {} not found".format(username))
                return -1, None, None
            elif not user.active:
                self.logger.warning("Mumble auth: User {} is deactivated".format(username))
                return -1, None, None

            if self.app.config.get("OTS_ENABLE_LDAP"):
                auth_result = ldap_manager.authenticate(username, password)
                if auth_result.status == AuthenticationResponseStatus.success:
                    self.logger.info("Mumble auth: {} has been authenticated".format(username))

                    # Keep this import here to avoid a circular import when OTS is started
                    from opentakserver.blueprints.ots_api.ldap_api import save_user

                    save_user(
                        auth_result.user_dn,
                        auth_result.user_id,
                        auth_result.user_info,
                        auth_result.user_groups,
                    )

                    return user.id, user.username, None

            elif verify_password(password, user.password):
                self.logger.info("Mumble auth: {} has been authenticated".format(username))
                return user.id, user.username, None

            self.logger.warning("Mumble auth: Bad password for {}".format(username))
            return -1, None, None

    def idToTexture(self, id, current=None):
        return

    def getInfo(self, id, current=None):
        """
        Gets called to fetch user specific information
        """

        # We do not expose any additional information so always fall through
        return False, None

    def nameToId(self, name, current=None):
        """
        Gets called to get the id for a given username
        """
        pass

    def idToName(self, id, current=None):
        """
        Gets called to get the username for a given id
        """
        pass

    def idToTexture(self, id, current=None):
        """
        Gets called to get the corresponding texture for a user
        """
        # seems like it pulled a user's avatar from a phpbb DB

    def registerUser(self, name, current=None):
        """
        Gets called when the server is asked to register a user.
        """
        pass

    def unregisterUser(self, id, current=None):
        """
        Gets called when the server is asked to unregister a user.
        """
        pass

    def getRegisteredUsers(self, filter, current=None):
        """
        Returns a list of usernames in the phpBB3 database which contain
        filter as a substring.
        """
        pass

    def setInfo(self, id, info, current=None):
        """
        Gets called when the server is supposed to save additional information
        about a user to his database
        """
        pass

    def setTexture(self, id, texture, current=None):
        """
        Gets called when the server is asked to update the user texture of a user
        """
        pass



import base64
import hashlib
import os.path
from datetime import datetime, timezone
from pathlib import Path
from xml.etree.ElementTree import tostring

from androguard.core.apk import APK
from bs4 import BeautifulSoup
from flask import current_app as app
from flask import request
from sqlalchemy import Boolean, DateTime, Integer, LargeBinary, String
from sqlalchemy.orm import Mapped, mapped_column
from werkzeug.utils import secure_filename

from opentakserver.extensions import db
from opentakserver.forms.package_form import PackageForm
from opentakserver.functions import iso8601_string_from_datetime


class Packages(db.Model):
    __tablename__ = "packages"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    platform: Mapped[str] = mapped_column(String(255))
    plugin_type: Mapped[str] = mapped_column(String(255))
    package_name: Mapped[str] = mapped_column(String(255))
    name: Mapped[str] = mapped_column(String(255))
    file_name: Mapped[str] = mapped_column(String(255))
    version: Mapped[str] = mapped_column(String(255))
    revision_code: Mapped[int] = mapped_column(Integer, nullable=True)
    description: Mapped[str] = mapped_column(String(255), nullable=True)
    apk_hash: Mapped[str] = mapped_column(String(255), nullable=True)
    os_requirement: Mapped[str] = mapped_column(Integer, nullable=True)
    tak_prereq: Mapped[str] = mapped_column(Integer, nullable=True)
    file_size: Mapped[int] = mapped_column(Integer)
    icon: Mapped[bytes] = mapped_column(LargeBinary, nullable=True)
    icon_filename: Mapped[str] = mapped_column(String(255), nullable=True)
    install_on_enrollment: Mapped[bool] = mapped_column(Boolean, default=False, nullable=True)
    install_on_connection: Mapped[bool] = mapped_column(Boolean, default=False, nullable=True)
    publish_time: Mapped[datetime] = mapped_column(DateTime)
    atak_version: Mapped[str] = mapped_column(String(255), nullable=True)

    def from_wtform(self, form: PackageForm):
        self.platform = form.platform.data
        self.plugin_type = form.plugin_type.data
        self.file_name = secure_filename(form.apk.data.filename)
        apk = APK(os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", self.file_name))
        self.package_name = apk.get_package()
        self.version = apk.get_androidversion_name()
        self.revision_code = apk.get_androidversion_code()
        self.os_requirement = apk.get_min_sdk_version()
        self.name = apk.get_app_name()
        self.description = form.description.data
        self.apk_hash = hashlib.sha256(form.apk.data.stream.read()).hexdigest()
        self.file_size = (
            Path(os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", self.file_name))
            .stat()
            .st_size
        )
        self.icon = request.files["icon"].stream.read() if "icon" in request.files else None
        self.icon_filename = (
            secure_filename(request.files["icon"].filename) if "icon" in request.files else None
        )
        self.install_on_enrollment = form.install_on_enrollment.data
        self.install_on_connection = form.install_on_connection.data
        self.publish_time = datetime.now(timezone.utc)
        self.atak_version = form.atak_version.data

        manifest = BeautifulSoup(tostring(apk.get_android_manifest_xml()).decode("utf-8"))
        meta_data = manifest.find_all("meta-data", "lxml")
        for meta in meta_data:
            if "ns0:value" not in meta.attrs or "ns0:name" not in meta.attrs:
                continue

            if meta.attrs["ns0:name"] == "plugin-api":
                self.tak_prereq = meta.attrs["ns0:value"]
                break

        if not self.icon:
            icon_filename, icon_extension = os.path.splitext(apk.get_app_icon())
            if icon_extension == ".png":
                self.icon = apk.get_file(apk.get_app_icon())
                self.icon_filename = f"{self.package_name}.png"

    def from_tak_gov(self, plugin: dict):
        self.apk_hash = plugin["apk_hash"]
        self.file_size = plugin["apk_size_bytes"]

        self.platform = plugin["platform"]

    def serialize(self):
        return {
            "platform": self.platform,
            "plugin_type": self.plugin_type,
            "package_name": self.package_name,
            "name": self.name,
            "file_name": self.file_name,
            "version": self.version,
            "revision_code": self.revision_code,
            "description": self.description,
            "apk_hash": self.apk_hash,
            "os_requirement": self.os_requirement,
            "tak_prereq": self.tak_prereq,
            "file_size": self.file_size,
            "icon": self.icon,
            "icon_filename": self.icon_filename,
            "install_on_enrollment": self.install_on_enrollment,
            "install_on_connection": self.install_on_connection,
            "publish_time": self.publish_time,
            "atak_version": self.atak_version,
        }

    def to_json(self):
        return_value = self.serialize()
        return_value["publish_time"] = iso8601_string_from_datetime(self.publish_time)
        return_value["icon"] = (
            f"data:image/png;base64,{base64.b64encode(self.icon).decode('utf-8')}"
            if self.icon
            else None
        )
        return return_value



import csv
import datetime
import os
import zipfile

import bleach
import sqlalchemy.exc
from flask import Blueprint
from flask import current_app as app
from flask import jsonify, request, send_from_directory
from flask_security import auth_required, roles_accepted
from werkzeug.datastructures import ImmutableMultiDict
from werkzeug.utils import secure_filename

from opentakserver.blueprints.marti_api.certificate_enrollment_api import basic_auth
from opentakserver.blueprints.marti_api.marti_api import verify_client_cert
from opentakserver.blueprints.ots_api.api import paginate, search
from opentakserver.extensions import db, logger
from opentakserver.forms.package_form import PackageForm, PackageUpdateForm
from opentakserver.models.Packages import Packages

packages_blueprint = Blueprint("packages_api_blueprint", __name__)


@packages_blueprint.route("/api/packages/<package_name>")
@packages_blueprint.route("/api/packages/<atak_version>/<package_name>")
def download_package(package_name, atak_version=None):
    cert = verify_client_cert()
    if not cert:
        return "", 401
    return send_from_directory(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages"), secure_filename(package_name)
    )


@packages_blueprint.route("/api/packages")
@auth_required()
@roles_accepted("administrator")
def get_packages():
    query = db.session.query(Packages)
    query = search(query, Packages, "platform")
    query = search(query, Packages, "plugin_type")
    query = search(query, Packages, "package_name")
    query = search(query, Packages, "name")
    query = search(query, Packages, "file_name")
    query = search(query, Packages, "version")
    query = search(query, Packages, "revision_code")
    query = search(query, Packages, "apk_hash")
    query = search(query, Packages, "tak_prereq")
    query = search(query, Packages, "file_size")
    query = search(query, Packages, "atak_version")

    return paginate(query)


@packages_blueprint.route("/api/packages/product.infz", methods=["HEAD"])
def head_product_infz():
    cert = verify_client_cert()
    if not cert:
        return "", 401

    if os.path.exists(os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", "product.infz")):
        return jsonify({"success": True})

    return jsonify({"success": False}), 404


@packages_blueprint.route("/api/packages/<atak_version>/product.infz", methods=["HEAD"])
def head_product_infz_with_version(atak_version: str):
    cert = verify_client_cert()
    if not cert:
        return "", 401

    if os.path.exists(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", atak_version, "product.infz")
    ):
        return jsonify({"success": True})

    return jsonify({"success": False}), 404


@packages_blueprint.route("/api/packages/product.infz", methods=["GET"])
def get_product_infz():
    cert = verify_client_cert()
    if not cert:
        return "", 401
    return send_from_directory(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages"), "product.infz"
    )


@packages_blueprint.route("/api/packages/<atak_version>/product.infz", methods=["GET"])
def get_product_infz_with_version(atak_version: str):
    cert = verify_client_cert()
    if not cert:
        return "", 401
    return send_from_directory(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", atak_version), "product.infz"
    )


def create_product_infz(atak_version: str | None):
    product_infz_file = os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", "product.infz")
    product_inf_file = os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", "product.inf")

    if atak_version:
        atak_version = bleach.clean(atak_version)
        product_infz_file = os.path.join(
            app.config.get("OTS_DATA_FOLDER"), "packages", atak_version, "product.infz"
        )
        product_inf_file = os.path.join(
            app.config.get("OTS_DATA_FOLDER"), "packages", atak_version, "product.inf"
        )
        os.makedirs(
            os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", atak_version), exist_ok=True
        )

    if os.path.exists(product_infz_file):
        os.remove(product_infz_file)

    query = db.session.query(Packages)
    if atak_version:
        query = query.where(Packages.atak_version == atak_version)
    packages = db.session.execute(query).all()

    with zipfile.ZipFile(product_infz_file, mode="a", compression=zipfile.ZIP_DEFLATED) as zipf:

        with open(product_inf_file, "w") as inf:
            csv_writer = csv.writer(inf)

            for package in packages:
                package = package[0]
                csv_writer.writerow(
                    [
                        package.platform,
                        package.plugin_type,
                        package.package_name,
                        package.name,
                        package.version,
                        package.revision_code,
                        package.file_name,
                        package.icon_filename,
                        package.description,
                        package.apk_hash,
                        package.os_requirement,
                        package.tak_prereq,
                        package.file_size,
                    ]
                )

                if package.icon:
                    zipf.writestr(package.icon_filename, package.icon)

        zipf.write(product_inf_file, arcname="product.inf")
        os.remove(product_inf_file)


@packages_blueprint.route("/api/packages/repositories.inf")
def get_repository_inf():
    cert = verify_client_cert()
    if not cert:
        return "", 401

    versions = (
        Packages.query.distinct(Packages.atak_version)
        .where(Packages.atak_version is not None)
        .all()
    )
    if not versions:
        return "", 404

    response = ""
    for version in versions:
        if version.atak_version:
            response += version.atak_version + "\n"

    return response, 200


@packages_blueprint.route("/api/packages", methods=["POST"])
@auth_required()
@roles_accepted("administrator")
def add_package():
    form = PackageForm()
    if not form.validate():
        return jsonify({"success": False, "errors": form.errors}), 400

    os.makedirs(os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages"), exist_ok=True)

    apk_filename = secure_filename(request.files["apk"].filename)

    request.files["apk"].save(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", apk_filename)
    )

    package = Packages()
    package.from_wtform(form)

    existing_package = db.session.execute(
        db.session.query(Packages).filter_by(
            package_name=package.package_name, atak_version=package.atak_version
        )
    ).scalar()
    if existing_package:
        logger.warning(
            f"{package.name} version {package.version} for ATAK {package.atak_version} is already on the server and will be removed"
        )

        if os.path.exists(
            os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", existing_package.file_name)
        ):
            os.remove(
                os.path.join(
                    app.config.get("OTS_DATA_FOLDER"), "packages", existing_package.file_name
                )
            )

        db.session.delete(existing_package)

    try:
        db.session.add(package)
        db.session.commit()
    except sqlalchemy.exc.IntegrityError:
        db.session.rollback()
        db.session.execute(
            sqlalchemy.update(Packages)
            .where(Packages.package_name == package.package_name)
            .values(**package.serialize())
        )
        db.session.commit()

    create_product_infz(form.atak_version.data)

    return jsonify({"success": True})


@packages_blueprint.route("/api/packages", methods=["PATCH"])
@auth_required()
@roles_accepted("administrator")
def edit_package():
    form = PackageUpdateForm(formdata=ImmutableMultiDict(request.json))
    if not form.validate():
        return jsonify({"success": False, "errors": form.errors}), 400

    package = db.session.execute(
        db.session.query(Packages).filter_by(
            package_name=form.package_name.data, atak_version=form.atak_version.data
        )
    ).first()
    if not package:
        return jsonify({"success": False, "error": f"{form.package_name.data} not found"}), 404

    package = package[0]
    package.install_on_enrollment = form.install_on_enrollment.data
    package.install_on_connection = form.install_on_connection.data
    package.publish_time = datetime.datetime.now(datetime.timezone.utc)

    db.session.execute(
        sqlalchemy.update(Packages)
        .where(Packages.package_name == form.package_name.data)
        .where(Packages.atak_version == form.atak_version.data)
        .values(**package.serialize())
    )
    db.session.commit()

    return jsonify({"success": True})


@packages_blueprint.route("/api/packages", methods=["DELETE"])
@auth_required()
@roles_accepted("administrator")
def delete_package():
    package_name = request.args.get("package_name")
    if not package_name:
        return (
            jsonify(
                {
                    "success": False,
                    "error": "Please provide the package name of the plugin to delete",
                }
            ),
            400,
        )

    atak_version = None

    query = db.session.query(Packages)
    query = search(query, Packages, "package_name")
    if request.args.get("atak_version"):
        atak_version = bleach.clean(request.args.get("atak_version"))
        query = query.where(Packages.atak_version == atak_version)

    package = db.session.execute(query).first()
    if not package:
        return jsonify({"success": False, "error": f"Unknown package name: {package_name}"}), 404

    package = package[0]
    if os.path.exists(
        os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", package.file_name)
    ):
        os.remove(os.path.join(app.config.get("OTS_DATA_FOLDER"), "packages", package.file_name))

    db.session.delete(package)
    db.session.commit()

    create_product_infz(atak_version)

    return jsonify({"success": True})



import hashlib
import json
import os
import time
import traceback
from dataclasses import dataclass

import jwt
from flask import current_app as app
from sqlalchemy import BigInteger, Boolean, ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db, logger


@dataclass
class Token(db.Model):
    __tablename__ = "tokens"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    username: Mapped[str] = mapped_column(
        String(255), ForeignKey("user.username"), nullable=True, unique=True
    )
    max_uses: Mapped[int] = mapped_column(Integer, default=None, nullable=True)
    total_uses: Mapped[int] = mapped_column(Integer, default=0, nullable=True)
    creation: Mapped[int] = mapped_column(BigInteger, nullable=False, default=int(time.time()))
    not_before: Mapped[int] = mapped_column(BigInteger, nullable=True)
    expiration: Mapped[int] = mapped_column(BigInteger, nullable=True)
    disabled: Mapped[bool] = mapped_column(Boolean, default=False, nullable=True)
    token_hash: Mapped[str] = mapped_column(String(255), nullable=False, unique=True)

    user = relationship("User", back_populates="tokens")

    def to_json(self, hash=False) -> dict:
        json_token = {
            "sub": self.username,
            "iat": self.creation,
            "iss": "OpenTAKServer",
            "aud": "OpenTAKServer",
        }

        if self.max_uses or not hash:
            json_token["max"] = self.max_uses

        if self.not_before or not hash:
            json_token["nbf"] = self.not_before

        if self.expiration or not hash:
            json_token["exp"] = self.expiration

        return json_token

    def hash_token(self, token: str = None) -> str:
        sha256 = hashlib.sha256()

        if token:
            sha256.update(token.encode())
            token_hash = sha256.hexdigest()
        else:
            sha256.update(json.dumps(self.to_json(True)).encode())
            token_hash = sha256.hexdigest()
            self.token_hash = token_hash

        return token_hash

    def generate_token(self) -> str or None:
        if not self.username:
            return None

        token = {
            "sub": self.username,
            "iat": self.creation,
            "iss": "OpenTAKServer",
            "aud": "OpenTAKServer",
        }

        if self.max_uses:
            token["max"] = self.max_uses

        if self.not_before:
            token["nbf"] = self.not_before

        if self.expiration:
            token["exp"] = self.expiration

        with open(
            os.path.join(
                app.config.get("OTS_CA_FOLDER"),
                "certs",
                "opentakserver",
                "opentakserver.nopass.key",
            ),
            "rb",
        ) as key:
            encoded_token = jwt.encode(token, key.read(), algorithm="RS256")
            return encoded_token

    @staticmethod
    def verify_token(token: str) -> bool:
        with open(
            os.path.join(
                app.config.get("OTS_CA_FOLDER"), "certs", "opentakserver", "opentakserver.pub"
            ),
            "r",
        ) as key:
            try:
                # Will raise InvalidTokenError on bad signature, expired, or before the nbf date
                decoded_token: dict = jwt.decode(
                    token, key.read(), algorithms=["RS256"], audience="OpenTAKServer"
                )

                sha256 = hashlib.sha256()
                sha256.update(json.dumps(decoded_token).encode())
                token_hash = sha256.hexdigest()

                token_from_db = db.session.query(Token).filter_by(token_hash=token_hash).first()
                if not token_from_db:
                    logger.error(f"Token not in db: {token_hash}")
                    return False

                if token_from_db.disabled:
                    logger.error("Token disabled")
                    return False

                if (
                    "max" in decoded_token.keys()
                    and token_from_db.total_uses >= decoded_token["max"]
                ):
                    logger.error(f"Too many uses for token {token_hash}")
                    return False

                token_from_db.total_uses += 1
                db.session.add(token_from_db)
                db.session.commit()

                return True

            except jwt.exceptions.InvalidTokenError as e:
                logger.error(f"Invalid token: {e}")
                logger.debug(traceback.format_exc())
                return False
            except BaseException as e:
                logger.error(f"Failed to decode token: {e}")
                logger.debug(traceback.format_exc())
                return False



from flask import Blueprint
from flask import current_app as app
from flask import jsonify, session
from flask_babel import gettext

from opentakserver.extensions import babel

language_api = Blueprint("language_api", __name__)


@language_api.route("/api/language/test")
def test_lang():
    return jsonify({"success": True, "message": gettext("This is a test")})


@language_api.route("/api/language")
def get_languages():
    return jsonify(app.config.get("OTS_LANGUAGES"))


@language_api.route("/api/language/<lang_code>", methods=["PUT"])
def set_language(lang_code):
    if not lang_code:
        session["language"] = "en"

    else:
        lang_codes = []
        for translation in babel.list_translations():
            lang_codes.append(translation.language)

        if lang_code in lang_codes:
            session["language"] = lang_code
            return jsonify({"success": True})
        else:
            return (
                jsonify(
                    {"success": False, "error": gettext(f"{lang_code} is not a supported language")}
                ),
                400,
            )



"""Added last_meshtastic_publish to EUDs table

Revision ID: 6665d030f187
Revises: 1041fae84708
Create Date: 2025-01-30 14:07:58.600407

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "6665d030f187"
down_revision = "1041fae84708"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.add_column(sa.Column("last_meshtastic_publish", sa.DateTime(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.drop_column("last_meshtastic_publish")

    # ### end Alembic commands ###



from sqlalchemy import ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db

from .GeoChat import GeoChat


class Chatroom(db.Model):
    __tablename__ = "chatrooms"

    id: Mapped[str] = mapped_column(String(255), primary_key=True)
    name: Mapped[str] = mapped_column(String(255))
    group_owner: Mapped[str] = mapped_column(String(255), nullable=True)
    parent: Mapped[str] = mapped_column(String(255), nullable=True)
    geochats = relationship("GeoChat", cascade="all, delete-orphan", back_populates="chatroom")
    chatroom_uid = relationship(
        "ChatroomsUids", cascade="all, delete-orphan", back_populates="chatroom"
    )
    team = relationship("Team", back_populates="chatroom")

    def serialize(self):
        return {
            "name": self.name,
            "group_owner": self.group_owner,
            "parent": self.parent,
        }

    def to_json(self):
        return {
            "name": self.name,
            "group_owner": self.group_owner,
            "parent": self.parent,
            "geochats": [chat.to_json() for chat in self.geochats] if self.geochats else None,
        }



import datetime
import uuid
from dataclasses import dataclass
from xml.etree.ElementTree import Element, SubElement

from bs4 import BeautifulSoup
from sqlalchemy import Boolean, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db, logger
from opentakserver.functions import iso8601_string_from_datetime
from opentakserver.models.Mission import Mission
from opentakserver.models.MissionContent import MissionContent
from opentakserver.models.MissionUID import MissionUID


@dataclass
class MissionChange(db.Model):
    __tablename__ = "mission_changes"

    CREATE_MISSION = "CREATE_MISSION"
    DELETE_MISSION = "DELETE_MISSION"
    ADD_CONTENT = "ADD_CONTENT"
    REMOVE_CONTENT = "REMOVE_CONTENT"
    CREATE_DATA_FEED = "CREATE_DATA_FEED"
    DELETE_DATA_FEED = "DELETE_DATA_FEED"
    CHANGE = "CHANGE"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    content_uid: Mapped[str] = mapped_column(
        String(255), ForeignKey("mission_content.uid"), nullable=True
    )
    isFederatedChange: Mapped[bool] = mapped_column(Boolean)
    change_type: Mapped[str] = mapped_column(String(255))
    mission_name: Mapped[str] = mapped_column(String(255), ForeignKey("missions.name"))
    timestamp: Mapped[datetime] = mapped_column(DateTime)
    creator_uid: Mapped[str] = mapped_column(String(255))
    server_time: Mapped[datetime] = mapped_column(DateTime)
    mission_uid: Mapped[str] = mapped_column(
        String(255), ForeignKey("mission_uids.uid", ondelete="CASCADE"), nullable=True
    )
    content_resource = relationship(
        "MissionContent", back_populates="mission_changes", uselist=False
    )
    mission = relationship("Mission", back_populates="mission_changes")
    uid = relationship("MissionUID", back_populates="mission_change", uselist=False)

    def serialize(self):
        return {
            "isFederatedChange": self.isFederatedChange,
            "change_type": self.change_type,
            "mission_name": self.mission_name,
            "timestamp": self.timestamp,
            "creator_uid": self.creator_uid,
            "server_time": self.server_time,
            "mission_uid": self.mission_uid,
            "content_uid": self.content_uid,
        }

    def to_json(self):
        json = {
            "isFederatedChange": self.isFederatedChange,
            "type": self.change_type,
            "contentUid": self.content_uid,
            "missionName": self.mission_name,
            "timestamp": iso8601_string_from_datetime(self.timestamp),
            "creatorUid": self.creator_uid if self.creator_uid else "",
            "serverTime": iso8601_string_from_datetime(self.server_time),
            "missionGuid": self.mission_uid,
        }

        if self.content_resource:
            json["contentResource"] = self.content_resource.to_json()["data"]

        if self.uid:
            json["details"] = self.uid.to_details_json()

        return json


def generate_mission_change_cot(
    author_uid: str,
    mission: Mission,
    mission_change: MissionChange,
    content: MissionContent | None = None,
    cot_event: BeautifulSoup | None = None,
    mission_uid: MissionUID = None,
    cot_type: str = "t-x-m-c",
) -> Element:
    if content:
        uid = content.uid
    elif cot_event:
        uid = cot_event.attrs["uid"]
    else:
        uid = str(uuid.uuid4())

    event = Element(
        "event",
        {
            "version": "2.0",
            "uid": uid,
            "type": cot_type,
            "how": "h-g-i-g-o",
            "start": iso8601_string_from_datetime(mission_change.timestamp),
            "time": iso8601_string_from_datetime(mission_change.timestamp),
            "stale": iso8601_string_from_datetime(
                mission_change.timestamp + datetime.timedelta(minutes=2)
            ),
            "access": "Undefined",
        },
    )
    SubElement(
        event, "point", {"ce": "9999999", "le": "9999999", "hae": "0.0", "lat": "0.0", "lon": "0.0"}
    )

    detail = SubElement(event, "detail")
    mission_element = SubElement(
        detail,
        "mission",
        {
            "type": mission_change.change_type,
            "tool": "public",
            "name": mission.name,
            "guid": mission.guid,
            "authorUid": mission_change.creator_uid,
        },
    )
    mission_changes_element = SubElement(mission_element, "MissionChanges")
    mission_change_element = SubElement(mission_changes_element, "MissionChange")

    if content:
        content_resource = SubElement(mission_change_element, "contentResource")
        SubElement(content_resource, "creatorUid").text = mission_change.creator_uid
        SubElement(content_resource, "expiration").text = "-1"
        SubElement(content_resource, "groupVector").text = "0"
        SubElement(content_resource, "hash").text = content.hash
        SubElement(content_resource, "mimeType").text = content.mime_type
        SubElement(content_resource, "name").text = content.filename
        SubElement(content_resource, "size").text = str(content.size)
        SubElement(content_resource, "submissionTime").text = iso8601_string_from_datetime(
            content.submission_time
        )
        SubElement(content_resource, "submitter").text = content.submitter
        SubElement(content_resource, "uid").text = content.uid
        SubElement(mission_change_element, "contentUid").text = mission_change.content_uid

    if cot_event:
        details_tag = SubElement(
            mission_change_element, "details", {"type": cot_event.attrs["type"]}
        )

        point = cot_event.find("point")
        color = cot_event.find("color")
        callsign = cot_event.find("contact")
        icon = cot_event.find("usericon")

        if color and "argb" in color.attrs:
            details_tag.set("color", color.attrs["argb"])
        if color and "value" in color.attrs:
            details_tag.set("color", color.attrs["value"])
        if callsign:
            details_tag.set("callsign", callsign.attrs["callsign"])
        if icon:
            details_tag.set("iconsetPath", icon.attrs["iconsetpath"])

        SubElement(details_tag, "location", {"lon": point.attrs["lon"], "lat": point.attrs["lat"]})
        # SubElement(mission_change_element, "contentUid").text = cot_event.attrs['uid']

    if mission_uid:
        details_tag = SubElement(mission_change_element, "details")
        if mission_uid.color:
            details_tag.set("color", str(mission_uid.color))
        if mission_uid.callsign:
            details_tag.set("callsign", mission_uid.callsign)
        if mission_uid.cot_type:
            details_tag.set("type", mission_uid.cot_type)
        if mission_uid.iconset_path:
            details_tag.set("iconsetPath", mission_uid.iconset_path)
        if mission_uid.longitude:
            SubElement(
                details_tag,
                "location",
                {"lon": str(mission_uid.longitude), "lat": str(mission_uid.latitude)},
            )

    if mission_change.mission_uid:
        SubElement(mission_change_element, "contentUid").text = mission_change.mission_uid

    SubElement(mission_change_element, "missionGuid").text = mission.guid
    SubElement(mission_change_element, "creatorUid").text = mission_change.creator_uid
    SubElement(mission_change_element, "isFederatedChange").text = str(
        mission_change.isFederatedChange
    ).lower()
    SubElement(mission_change_element, "missionName").text = mission.name
    SubElement(mission_change_element, "timestamp").text = iso8601_string_from_datetime(
        mission_change.timestamp
    )
    SubElement(mission_change_element, "type").text = mission_change.change_type

    return event



"""Changed phone_number column to BigInteger for MySQL compatibility

Revision ID: 7e9f5d2c193d
Revises: 8787888a028f
Create Date: 2024-10-25 17:37:42.637254

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "7e9f5d2c193d"
down_revision = "8787888a028f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.alter_column("callsign", existing_type=sa.String(length=255), nullable=True)
        batch_op.alter_column(
            "phone_number",
            existing_type=sa.INTEGER(),
            type_=sa.BigInteger(),
            existing_nullable=True,
        )
    with op.batch_alter_table("data_packages", schema=None) as batch_op:
        batch_op.alter_column("creator_uid", existing_type=sa.String(length=255), nullable=True)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.alter_column(
            "phone_number",
            existing_type=sa.BigInteger(),
            type_=sa.INTEGER(),
            existing_nullable=True,
        )
        batch_op.alter_column("callsign", existing_type=sa.String(length=255), nullable=True)
    with op.batch_alter_table("data_packages", schema=None) as batch_op:
        batch_op.alter_column("creator_uid", existing_type=sa.String(length=255), nullable=False)

    # ### end Alembic commands ###



"""Update for group support

Revision ID: e8bd877719e2
Revises: 3a7ea7ead17b
Create Date: 2025-10-08 18:56:33.910860

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "e8bd877719e2"
down_revision = "3a7ea7ead17b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "groups_users",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("group_id", sa.Integer(), nullable=False),
        sa.Column("direction", sa.String(length=255), nullable=False),
        sa.Column("enabled", sa.Boolean(), nullable=False),
        sa.ForeignKeyConstraint(
            ["group_id"],
            ["groups.id"],
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["user.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.drop_table("groups_euds")

    with op.batch_alter_table("groups", schema=None) as batch_op:
        batch_op.add_column(sa.Column("name", sa.String(length=255), nullable=False))
        batch_op.add_column(sa.Column("distinguishedName", sa.String(length=255), nullable=True))
        batch_op.add_column(sa.Column("type", sa.String(length=255), nullable=False))
        batch_op.add_column(sa.Column("description", sa.String(length=255), nullable=True))
        batch_op.drop_column("group_type")
        batch_op.drop_column("group_name")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("groups", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("group_name", sa.VARCHAR(length=255), autoincrement=False, nullable=False)
        )
        batch_op.add_column(
            sa.Column("group_type", sa.VARCHAR(length=255), autoincrement=False, nullable=False)
        )
        batch_op.drop_column("description")
        batch_op.drop_column("type")
        batch_op.drop_column("distinguishedName")
        batch_op.drop_column("name")

    op.create_table(
        "groups_euds",
        sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False),
        sa.Column("group_id", sa.INTEGER(), autoincrement=False, nullable=False),
        sa.Column("direction", sa.VARCHAR(length=255), autoincrement=False, nullable=False),
        sa.Column("active", sa.BOOLEAN(), autoincrement=False, nullable=False),
        sa.Column("description", sa.VARCHAR(length=255), autoincrement=False, nullable=True),
        sa.Column("eud_uid", sa.VARCHAR(length=255), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["eud_uid"], ["euds.uid"], name=op.f("group_eud"), ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["group_id"], ["groups.id"], name=op.f("group")),
        sa.PrimaryKeyConstraint("id", name=op.f("groups_euds_pkey")),
    )
    op.drop_table("groups_users")
    # ### end Alembic commands ###



"""Added groups table

Revision ID: d4cc3d4afdb9
Revises: d1f5df78eace
Create Date: 2024-10-15 19:03:59.996651

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "d4cc3d4afdb9"
down_revision = "d1f5df78eace"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "groups",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("group_name", sa.String(length=255), nullable=False),
        sa.Column("direction", sa.String(length=255), nullable=False),
        sa.Column("created", sa.DateTime(), nullable=False),
        sa.Column("group_type", sa.String(length=255), nullable=False),
        sa.Column("bitpos", sa.Integer(), nullable=False),
        sa.Column("active", sa.Boolean(), nullable=False),
        sa.Column("description", sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    op.drop_table("groups")
    # ### end Alembic commands ###



from sqlalchemy import Boolean, Float, ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db
from opentakserver.forms.zmist_form import ZmistForm


class ZMIST(db.Model):
    __tablename__ = "zmist"

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    i: Mapped[str] = mapped_column(String(255), nullable=True)  # injury_sustained
    m: Mapped[str] = mapped_column(String(255), nullable=True)  # mechanism_of_injury
    s: Mapped[str] = mapped_column(String(255), nullable=True)  # symptoms_and_signs
    t: Mapped[str] = mapped_column(String(255), nullable=True)  # treatment_given
    title: Mapped[str] = mapped_column(String(255), nullable=True)
    z: Mapped[int] = mapped_column(Integer, nullable=True)  # zap_number
    casevac_uid: Mapped[str] = mapped_column(
        String(255), ForeignKey("casevac.uid", ondelete="CASCADE")
    )
    casevac = relationship("CasEvac", back_populates="zmist")

    def from_wtform(self, form: ZmistForm):
        self.i = form.i.data
        self.m = form.m.data
        self.s = form.s.data
        self.t = form.t.data
        self.title = form.title.data
        self.z = form.z.data

    def serialize(self):
        return {
            "i": self.i,
            "m": self.m,
            "s": self.s,
            "t": self.t,
            "title": self.title,
            "z": self.z,
        }

    def to_json(self):
        return self.serialize()



from flask import Blueprint
from flask_security import auth_required

from opentakserver.blueprints.ots_api.api import paginate, search
from opentakserver.extensions import db
from opentakserver.models.EUDStats import EUDStats

eud_stats_blueprint = Blueprint("eud_stats_blueprint", __name__)


@eud_stats_blueprint.route("/api/eud_stats")
@auth_required()
def get_stats():

    query = db.session.query(EUDStats)
    query = search(query, EUDStats, "eud_uid")
    # TODO: Implement date range search
    # query = search(query, EUDStats, 'from')
    # query = search(query, EUDStats, 'to')

    rows = db.session.execute(query.order_by(EUDStats.id.desc()).limit(50)).all()

    results = {"results": [], "total_pages": 1, "current_page": 1, "per_page": len(rows)}

    for row in rows:
        results["results"].insert(0, row[0].to_json())

    return results



"""Fixes to support MariaDB

Revision ID: 8787888a028f
Revises: f6dfc571d31c
Create Date: 2024-10-25 15:44:27.370608

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "8787888a028f"
down_revision = "f6dfc571d31c"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.alter_column(
            "xml", existing_type=sa.String(length=255), type_=sa.TEXT(), existing_nullable=False
        )

    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.alter_column(
            "meshtastic_id",
            existing_type=sa.INTEGER(),
            type_=sa.BigInteger(),
            existing_nullable=True,
        )

    with op.batch_alter_table("points", schema=None) as batch_op:
        batch_op.alter_column("device_uid", existing_type=sa.String(length=255), nullable=True)

    with op.batch_alter_table("missions", schema=None) as batch_op:
        batch_op.create_foreign_key("mission_creator", "euds", ["creator_uid"], ["uid"])

    with op.batch_alter_table("groups", schema=None) as batch_op:
        batch_op.add_column(sa.Column("eud_uid", sa.String(length=255), nullable=False))
        batch_op.create_foreign_key("eud", "euds", ["eud_uid"], ["uid"])

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("missions", schema=None) as batch_op:
        batch_op.drop_constraint("mission_creator", type_="foreignkey")

    with op.batch_alter_table("euds", schema=None) as batch_op:
        batch_op.alter_column(
            "meshtastic_id",
            existing_type=sa.BigInteger(),
            type_=sa.INTEGER(),
            existing_nullable=True,
        )

    with op.batch_alter_table("cot", schema=None) as batch_op:
        batch_op.alter_column(
            "xml", existing_type=sa.TEXT(), type_=sa.String(length=255), existing_nullable=False
        )

    with op.batch_alter_table("groups", schema=None) as batch_op:
        batch_op.drop_constraint("eud", type_="foreignkey")
        batch_op.drop_column("eud_uid")

    with op.batch_alter_table("points", schema=None) as batch_op:
        batch_op.alter_column("device_uid", existing_type=sa.String(length=255), nullable=False)

    # ### end Alembic commands ###



"""Initial Migration

Revision ID: 4c7909c34d4e
Revises:
Create Date: 2024-05-24 04:08:57.447120

"""

import logging

import flask_security
import sqlalchemy as sa
from alembic import op

logger = logging.getLogger("OpenTAKServer")
logger.disabled = False


# revision identifiers, used by Alembic.
revision = "4c7909c34d4e"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "chatrooms",
        sa.Column("id", sa.String(length=255), nullable=False),
        sa.Column("name", sa.String(length=255), nullable=False),
        sa.Column("group_owner", sa.String(length=255), nullable=True),
        sa.Column("parent", sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "icons",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("iconset_uid", sa.String(length=255), nullable=True),
        sa.Column("filename", sa.String(length=255), nullable=True),
        sa.Column("groupName", sa.String(length=255), nullable=True),
        sa.Column("type2525b", sa.String(length=255), nullable=True),
        sa.Column("useCnt", sa.Integer(), nullable=True),
        sa.Column("bitmap", sa.LargeBinary(), nullable=True),
        sa.Column("shadow", sa.LargeBinary(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "role",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=80), nullable=False),
        sa.Column("description", sa.String(length=255), nullable=True),
        sa.Column("permissions", flask_security.datastore.AsaList(), nullable=True),
        sa.Column(
            "update_datetime",
            sa.DateTime(),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=False,
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )

    op.create_table(
        "user",
        sa.Column("email", sa.String(length=255), nullable=True),
        sa.Column("fs_webauthn_user_handle", sa.String(length=64), nullable=True),
        sa.Column("mf_recovery_codes", flask_security.datastore.AsaList(), nullable=True),
        sa.Column("password", sa.String(length=255), nullable=True),
        sa.Column("us_phone_number", sa.String(length=128), nullable=True),
        sa.Column("username", sa.String(length=255), nullable=True),
        sa.Column("us_totp_secrets", sa.Text(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("active", sa.Boolean(), nullable=False),
        sa.Column("fs_uniquifier", sa.String(length=64), nullable=False),
        sa.Column("confirmed_at", sa.DateTime(), nullable=True),
        sa.Column("last_login_at", sa.DateTime(), nullable=True),
        sa.Column("current_login_at", sa.DateTime(), nullable=True),
        sa.Column("last_login_ip", sa.String(length=64), nullable=True),
        sa.Column("current_login_ip", sa.String(length=64), nullable=True),
        sa.Column("login_count", sa.Integer(), nullable=True),
        sa.Column("tf_primary_method", sa.String(length=64), nullable=True),
        sa.Column("tf_totp_secret", sa.String(length=255), nullable=True),
        sa.Column("tf_phone_number", sa.String(length=128), nullable=True),
        sa.Column(
            "create_datetime",
            sa.DateTime(),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=False,
        ),
        sa.Column(
            "update_datetime",
            sa.DateTime(),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=False,
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("fs_uniquifier"),
        sa.UniqueConstraint("fs_webauthn_user_handle"),
        sa.UniqueConstraint("us_phone_number"),
        sa.UniqueConstraint("username"),
    )

    op.create_table(
        "roles_users",
        sa.Column("user_id", sa.Integer(), nullable=True),
        sa.Column("role_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["role_id"], ["role.id"], name="role"),
        sa.ForeignKeyConstraint(["user_id"], ["user.id"], name="user"),
    )

    op.create_table(
        "teams",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=255), nullable=False),
        sa.Column("chatroom_id", sa.String(length=255), nullable=True),
        sa.ForeignKeyConstraint(["chatroom_id"], ["chatrooms.id"], name="team_chatroom"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )

    op.create_table(
        "web_authn",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("credential_id", sa.LargeBinary(length=1024), nullable=False),
        sa.Column("public_key", sa.LargeBinary(), nullable=False),
        sa.Column("sign_count", sa.Integer(), nullable=True),
        sa.Column("transports", flask_security.datastore.AsaList(), nullable=True),
        sa.Column("backup_state", sa.Boolean(), nullable=False),
        sa.Column("device_type", sa.String(length=64), nullable=False),
        sa.Column("extensions", sa.String(length=255), nullable=True),
        sa.Column(
            "create_datetime",
            sa.DateTime(),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=False,
        ),
        sa.Column("lastuse_datetime", sa.DateTime(), nullable=False),
        sa.Column("name", sa.String(length=64), nullable=False),
        sa.Column("usage", sa.String(length=64), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["user_id"], ["user.id"], name="web_authn_user", ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    with op.batch_alter_table("web_authn", schema=None) as batch_op:
        batch_op.create_index(
            batch_op.f("ix_web_authn_credential_id"), ["credential_id"], unique=True
        )

    op.create_table(
        "euds",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("callsign", sa.String(length=255), unique=True),
        sa.Column("device", sa.String(length=255), nullable=True),
        sa.Column("os", sa.String(length=255), nullable=True),
        sa.Column("platform", sa.String(length=255), nullable=True),
        sa.Column("version", sa.String(length=255), nullable=True),
        sa.Column("phone_number", sa.Integer(), nullable=True),
        sa.Column("last_event_time", sa.DateTime(), nullable=True),
        sa.Column("last_status", sa.String(length=255), nullable=True),
        sa.Column("user_id", sa.Integer(), nullable=True),
        sa.Column("team_id", sa.Integer(), nullable=True),
        sa.Column("team_role", sa.String(length=255), nullable=True),
        sa.ForeignKeyConstraint(["team_id"], ["teams.id"], name="eud_team"),
        sa.ForeignKeyConstraint(["user_id"], ["user.id"], name="eud_user"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("uid"),
    )

    op.create_table(
        "chatrooms_uids",
        sa.Column("chatroom_id", sa.String(length=255), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.ForeignKeyConstraint(["chatroom_id"], ["chatrooms.id"], name="chatroom_uids"),
        sa.ForeignKeyConstraint(["uid"], ["euds.uid"], name="chatroom_eud"),
        sa.PrimaryKeyConstraint("chatroom_id", "uid"),
    )

    op.create_table(
        "cot",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("how", sa.String(length=255), nullable=True),
        sa.Column("type", sa.String(length=255), nullable=True),
        sa.Column("sender_callsign", sa.String(length=255), nullable=False),
        sa.Column("sender_device_name", sa.String(length=255), nullable=True),
        sa.Column("sender_uid", sa.String(length=255), nullable=True),
        sa.Column("recipients", sa.JSON(), nullable=True),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.Column("start", sa.DateTime(), nullable=False),
        sa.Column("stale", sa.DateTime(), nullable=False),
        sa.Column("xml", sa.TEXT, nullable=False),
        sa.ForeignKeyConstraint(["sender_uid"], ["euds.uid"], name="cot_eud"),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "data_packages",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("filename", sa.String(length=255), nullable=False),
        sa.Column("hash", sa.String(length=255), nullable=False),
        sa.Column("creator_uid", sa.String(length=255), nullable=False),
        sa.Column("submission_time", sa.DateTime(), nullable=False),
        sa.Column("submission_user", sa.Integer(), nullable=True),
        sa.Column("keywords", sa.String(length=255), nullable=True),
        sa.Column("mime_type", sa.String(length=255), nullable=False),
        sa.Column("size", sa.Integer(), nullable=False),
        sa.Column("tool", sa.String(length=255), nullable=True),
        sa.Column("expiration", sa.String(length=255), nullable=True),
        sa.ForeignKeyConstraint(["creator_uid"], ["euds.uid"], name="data_packages_eud"),
        sa.ForeignKeyConstraint(["submission_user"], ["user.id"], name="data_packages_user"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("filename"),
        sa.UniqueConstraint("hash"),
    )

    op.create_table(
        "certificates",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("common_name", sa.String(length=255), nullable=False),
        sa.Column("eud_uid", sa.String(length=255), nullable=True),
        sa.Column("data_package_id", sa.Integer(), nullable=True),
        sa.Column("callsign", sa.String(length=255), nullable=True),
        sa.Column("username", sa.String(length=255), nullable=True),
        sa.Column("expiration_date", sa.DateTime(), nullable=False),
        sa.Column("server_address", sa.String(length=255), nullable=False),
        sa.Column("server_port", sa.Integer(), nullable=False),
        sa.Column("truststore_filename", sa.String(length=255), nullable=False),
        sa.Column("user_cert_filename", sa.String(length=255), nullable=False),
        sa.Column("csr", sa.String(length=255), nullable=True),
        sa.Column("cert_password", sa.String(length=255), nullable=False),
        sa.ForeignKeyConstraint(
            ["data_package_id"], ["data_packages.id"], name="certificate_data_package"
        ),
        sa.ForeignKeyConstraint(["eud_uid"], ["euds.uid"], name="certificate_eud"),
        sa.ForeignKeyConstraint(["username"], ["user.username"], name="certificate_user"),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "points",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("device_uid", sa.String(length=255), nullable=False),
        sa.Column("latitude", sa.Float(), nullable=True),
        sa.Column("longitude", sa.Float(), nullable=True),
        sa.Column("ce", sa.Float(), nullable=True),
        sa.Column("hae", sa.Float(), nullable=True),
        sa.Column("le", sa.Float(), nullable=True),
        sa.Column("course", sa.Float(), nullable=True),
        sa.Column("speed", sa.Float(), nullable=True),
        sa.Column("location_source", sa.String(length=255), nullable=True),
        sa.Column("battery", sa.Float(), nullable=True),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.Column("azimuth", sa.Float(), nullable=True),
        sa.Column("fov", sa.Float(), nullable=True),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="point_cot"),
        sa.ForeignKeyConstraint(["device_uid"], ["euds.uid"], name="point_eud"),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "video_streams",
        sa.Column("path", sa.String(length=255), nullable=False),
        sa.Column("protocol", sa.String(length=255), nullable=False),
        sa.Column("port", sa.Integer(), nullable=False),
        sa.Column("network_timeout", sa.Integer(), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=True),
        sa.Column("buffer_time", sa.Integer(), nullable=True),
        sa.Column("rover_port", sa.Integer(), nullable=True),
        sa.Column("rtsp_reliable", sa.Integer(), nullable=True),
        sa.Column("ignore_embedded_klv", sa.Boolean(), nullable=True),
        sa.Column("alias", sa.String(length=255), nullable=True),
        sa.Column("preferred_mac_address", sa.String(length=255), nullable=True),
        sa.Column("preferred_interface_address", sa.String(length=255), nullable=True),
        sa.Column("username", sa.String(length=255), nullable=True),
        sa.Column("xml", sa.String(length=255), nullable=True),
        sa.Column("ready", sa.Boolean(), nullable=False),
        sa.Column("mediamtx_settings", sa.String(length=255), nullable=False),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="video_stream_cot"),
        sa.ForeignKeyConstraint(["username"], ["user.username"], name="video_stream_user"),
        sa.PrimaryKeyConstraint("path"),
    )

    op.create_table(
        "alerts",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("sender_uid", sa.String(length=255), nullable=False),
        sa.Column("start_time", sa.DateTime(), nullable=False),
        sa.Column("cancel_time", sa.DateTime(), nullable=True),
        sa.Column("alert_type", sa.String(length=255), nullable=False),
        sa.Column("point_id", sa.Integer(), nullable=True),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="alert_cot"),
        sa.ForeignKeyConstraint(["point_id"], ["points.id"], name="alert_point"),
        sa.ForeignKeyConstraint(["sender_uid"], ["euds.uid"], name="alert_eud"),
        sa.PrimaryKeyConstraint("id"),
    )

    op.create_table(
        "casevac",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("sender_uid", sa.String(length=255), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.Column("ambulatory", sa.Integer(), nullable=True),
        sa.Column("casevac", sa.Boolean(), nullable=True),
        sa.Column("child", sa.Integer(), nullable=True),
        sa.Column("enemy", sa.String(length=255), nullable=True),
        sa.Column("epw", sa.Integer(), nullable=True),
        sa.Column("equipment_detail", sa.String(length=255), nullable=True),
        sa.Column("equipment_none", sa.Boolean(), nullable=True),
        sa.Column("equipment_other", sa.Boolean(), nullable=True),
        sa.Column("extraction_equipment", sa.Boolean(), nullable=True),
        sa.Column("freq", sa.Float(), nullable=True),
        sa.Column("friendlies", sa.String(length=255), nullable=True),
        sa.Column("hlz_marking", sa.Integer(), nullable=True),
        sa.Column("hlz_remarks", sa.String(length=255), nullable=True),
        sa.Column("hoist", sa.Boolean(), nullable=True),
        sa.Column("litter", sa.Integer(), nullable=True),
        sa.Column("marked_by", sa.String(length=255), nullable=True),
        sa.Column("medline_remarks", sa.String(length=255), nullable=True),
        sa.Column("nonus_civilian", sa.Integer(), nullable=True),
        sa.Column("nonus_military", sa.Integer(), nullable=True),
        sa.Column("obstacles", sa.String(length=255), nullable=True),
        sa.Column("priority", sa.Integer(), nullable=True),
        sa.Column("routine", sa.Integer(), nullable=True),
        sa.Column("security", sa.Integer(), nullable=True),
        sa.Column("terrain_loose", sa.Boolean(), nullable=True),
        sa.Column("terrain_other", sa.Boolean(), nullable=True),
        sa.Column("terrain_other_detail", sa.String(length=255), nullable=True),
        sa.Column("terrain_detail", sa.String(length=255), nullable=True),
        sa.Column("terrain_none", sa.Boolean(), nullable=True),
        sa.Column("terrain_rough", sa.Boolean(), nullable=True),
        sa.Column("terrain_slope", sa.Boolean(), nullable=True),
        sa.Column("terrain_slope_dir", sa.String(length=255), nullable=True),
        sa.Column("title", sa.String(length=255), nullable=False),
        sa.Column("urgent", sa.Integer(), nullable=True),
        sa.Column("us_civilian", sa.Integer(), nullable=True),
        sa.Column("us_military", sa.Integer(), nullable=True),
        sa.Column("ventilator", sa.Boolean(), nullable=True),
        sa.Column("winds_are_from", sa.String(length=255), nullable=True),
        sa.Column("zone_prot_selection", sa.Integer(), nullable=True),
        sa.Column("point_id", sa.Integer(), nullable=True),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="casevac_cot"),
        sa.ForeignKeyConstraint(["point_id"], ["points.id"], name="casevac_point"),
        sa.ForeignKeyConstraint(["sender_uid"], ["euds.uid"], name="casevac_eud"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("uid"),
    )

    op.create_table(
        "geochat",
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("chatroom_id", sa.String(length=255), nullable=False),
        sa.Column("sender_uid", sa.String(length=255), nullable=False),
        sa.Column("remarks", sa.String(length=255), nullable=False),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.Column("point_id", sa.Integer(), nullable=False),
        sa.Column("cot_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["chatroom_id"], ["chatrooms.id"], name="geochat_chatroom"),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="geochat_cot"),
        sa.ForeignKeyConstraint(["point_id"], ["points.id"], name="geochat_point"),
        sa.ForeignKeyConstraint(["sender_uid"], ["euds.uid"], name="geochat_eud"),
        sa.PrimaryKeyConstraint("uid"),
    )

    op.create_table(
        "markers",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("affiliation", sa.String(length=255), nullable=True),
        sa.Column("battle_dimension", sa.String(length=255), nullable=True),
        sa.Column("point_id", sa.Integer(), nullable=False),
        sa.Column("callsign", sa.String(length=255), nullable=True),
        sa.Column("readiness", sa.Boolean(), nullable=True),
        sa.Column("argb", sa.Integer(), nullable=True),
        sa.Column("color_hex", sa.String(length=255), nullable=True),
        sa.Column("iconset_path", sa.String(length=255), nullable=True),
        sa.Column("parent_callsign", sa.String(length=255), nullable=True),
        sa.Column("production_time", sa.String(length=255), nullable=True),
        sa.Column("relation", sa.String(length=255), nullable=True),
        sa.Column("relation_type", sa.String(length=255), nullable=True),
        sa.Column("location_source", sa.String(length=255), nullable=True),
        sa.Column("icon_id", sa.Integer(), nullable=True),
        sa.Column("parent_uid", sa.String(length=255), nullable=True),
        sa.Column("remarks", sa.String(length=255), nullable=True),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.Column("mil_std_2525c", sa.String(length=255), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="marker_cot"),
        sa.ForeignKeyConstraint(["icon_id"], ["icons.id"], name="marker_icon"),
        sa.ForeignKeyConstraint(["parent_uid"], ["euds.uid"], name="marker_eud"),
        sa.ForeignKeyConstraint(["point_id"], ["points.id"], name="marker_point"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("uid"),
    )

    op.create_table(
        "rb_lines",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("sender_uid", sa.String(length=255), nullable=False),
        sa.Column("uid", sa.String(length=255), nullable=False),
        sa.Column("timestamp", sa.DateTime(), nullable=False),
        sa.Column("range", sa.Float(), nullable=False),
        sa.Column("bearing", sa.Float(), nullable=False),
        sa.Column("inclination", sa.Float(), nullable=True),
        sa.Column("anchor_uid", sa.String(length=255), nullable=True),
        sa.Column("range_units", sa.Integer(), nullable=True),
        sa.Column("bearing_units", sa.Integer(), nullable=True),
        sa.Column("north_ref", sa.Integer(), nullable=True),
        sa.Column("color", sa.Integer(), nullable=True),
        sa.Column("color_hex", sa.String(length=255), nullable=True),
        sa.Column("callsign", sa.String(length=255), nullable=True),
        sa.Column("stroke_color", sa.Integer(), nullable=True),
        sa.Column("stroke_weight", sa.Float(), nullable=True),
        sa.Column("stroke_style", sa.String(length=255), nullable=True),
        sa.Column("labels_on", sa.Boolean(), nullable=True),
        sa.Column("point_id", sa.Integer(), nullable=True),
        sa.Column("cot_id", sa.Integer(), nullable=True),
        sa.Column("end_latitude", sa.Float(), nullable=True),
        sa.Column("end_longitude", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(["cot_id"], ["cot.id"], name="rb_line_cot"),
        sa.ForeignKeyConstraint(["point_id"], ["points.id"], name="rb_line_point"),
        sa.ForeignKeyConstraint(["sender_uid"], ["euds.uid"], name="rb_line_eud"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("uid"),
    )

    op.create_table(
        "video_recordings",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("segment_path", sa.String(length=255), nullable=False),
        sa.Column("path", sa.String(length=255), nullable=False),
        sa.Column("in_progress", sa.Boolean(), nullable=False),
        sa.Column("start_time", sa.DateTime(), nullable=True),
        sa.Column("stop_time", sa.DateTime(), nullable=True),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("width", sa.Integer(), nullable=True),
        sa.Column("height", sa.Integer(), nullable=True),
        sa.Column("video_codec", sa.String(length=255), nullable=True),
        sa.Column("video_bitrate", sa.Integer(), nullable=True),
        sa.Column("audio_codec", sa.String(length=255), nullable=True),
        sa.Column("audio_bitrate", sa.Integer(), nullable=True),
        sa.Column("audio_samplerate", sa.Integer(), nullable=True),
        sa.Column("audio_channels", sa.Integer(), nullable=True),
        sa.Column("file_size", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["path"], ["video_streams.path"], name="video_recording_video_stream"
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("segment_path"),
    )

    op.create_table(
        "zmist",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("i", sa.String(length=255), nullable=True),
        sa.Column("m", sa.String(length=255), nullable=True),
        sa.Column("s", sa.String(length=255), nullable=True),
        sa.Column("t", sa.String(length=255), nullable=True),
        sa.Column("title", sa.String(length=255), nullable=True),
        sa.Column("z", sa.Integer(), nullable=True),
        sa.Column("casevac_uid", sa.String(length=255), nullable=False),
        sa.ForeignKeyConstraint(["casevac_uid"], ["casevac.uid"], name="zmist_casevac"),
        sa.PrimaryKeyConstraint("id"),
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("zmist")
    op.drop_table("video_recordings")
    op.drop_table("rb_lines")
    op.drop_table("markers")
    op.drop_table("geochat")
    op.drop_table("casevac")
    op.drop_table("alerts")
    op.drop_table("video_streams")
    op.drop_table("points")
    op.drop_table("certificates")
    op.drop_table("data_packages")
    op.drop_table("cot")
    op.drop_table("chatrooms_uids")
    op.drop_table("euds")
    with op.batch_alter_table("web_authn", schema=None) as batch_op:
        batch_op.drop_index(batch_op.f("ix_web_authn_credential_id"))

    op.drop_table("web_authn")
    op.drop_table("teams")
    op.drop_table("roles_users")
    op.drop_table("user")
    op.drop_table("role")
    op.drop_table("icons")
    op.drop_table("chatrooms")
    # ### end Alembic commands ###



from datetime import datetime

from sqlalchemy import DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db


class GeoChat(db.Model):
    __tablename__ = "geochat"

    uid: Mapped[str] = mapped_column(String(255), primary_key=True)
    chatroom_id: Mapped[str] = mapped_column(String(255), ForeignKey("chatrooms.id"))
    sender_uid: Mapped[str] = mapped_column(String(255), ForeignKey("euds.uid", ondelete="CASCADE"))
    remarks: Mapped[str] = mapped_column(String(255))
    timestamp: Mapped[datetime] = mapped_column(DateTime)
    point_id: Mapped[int] = mapped_column(Integer, ForeignKey("points.id"))
    cot_id: Mapped[int] = mapped_column(Integer, ForeignKey("cot.id"))
    point = relationship("Point", cascade="all", back_populates="geochat", uselist=False)
    cot = relationship("CoT", cascade="all", back_populates="geochat")
    chatroom = relationship("Chatroom", back_populates="geochats")
    eud = relationship("EUD", back_populates="geochats")

    def serialize(self):
        return {
            "uid": self.uid,
            "sender_uid": self.sender_uid,
            "remarks": self.remarks,
            "timestamp": self.timestamp,
        }

    def to_json(self):
        return {
            "uid": self.uid,
            "sender_uid": self.sender_uid,
            "remarks": self.remarks,
            "timestamp": self.timestamp,
            "point": self.point.to_json() or None,
        }



"""Changed device_profile table

Revision ID: 34dc96ee805b
Revises: 414b87c32cc2
Create Date: 2024-09-12 04:42:15.909042

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "34dc96ee805b"
down_revision = "414b87c32cc2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("device_profiles", schema=None) as batch_op:
        batch_op.add_column(sa.Column("id", sa.Integer(), nullable=False))
        batch_op.alter_column("preference_key", existing_type=sa.String(length=255), nullable=False)
        batch_op.alter_column(
            "preference_value", existing_type=sa.String(length=255), nullable=True
        )
        batch_op.alter_column("value_class", existing_type=sa.String(length=255), nullable=True)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("device_profiles", schema=None) as batch_op:
        batch_op.alter_column("value_class", existing_type=sa.String(length=255), nullable=False)
        batch_op.alter_column(
            "preference_value", existing_type=sa.String(length=255), nullable=False
        )
        batch_op.alter_column("preference_key", existing_type=sa.String(length=255), nullable=False)
        batch_op.drop_column("id")

    # ### end Alembic commands ###



import datetime
from dataclasses import dataclass

from sqlalchemy import Boolean, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import Mapped, mapped_column, relationship

from opentakserver.extensions import db
from opentakserver.functions import iso8601_string_from_datetime


@dataclass
class MissionRole(db.Model):
    __tablename__ = "mission_roles"

    MISSION_MANAGE_FEEDS = "MISSION_MANAGE_FEEDS"
    MISSION_SET_PASSWORD = "MISSION_SET_PASSWORD"
    MISSION_WRITE = "MISSION_WRITE"
    MISSION_MANAGE_LAYERS = "MISSION_MANAGE_LAYERS"
    MISSION_UPDATE_GROUPS = "MISSION_UPDATE_GROUPS"
    MISSION_DELETE = "MISSION_DELETE"
    MISSION_SET_ROLE = "MISSION_SET_ROLE"
    MISSION_READ = "MISSION_READ"
    MISSION_OWNER = "MISSION_OWNER"
    MISSION_SUBSCRIBER = "MISSION_SUBSCRIBER"
    MISSION_READ_ONLY = "MISSION_READ_ONLY"

    OWNER_ROLE = {"type": MISSION_OWNER, "permissions": []}
    OWNER_ROLE["permissions"].append(MISSION_MANAGE_FEEDS)
    OWNER_ROLE["permissions"].append(MISSION_SET_PASSWORD)
    OWNER_ROLE["permissions"].append(MISSION_WRITE)
    OWNER_ROLE["permissions"].append(MISSION_MANAGE_LAYERS)
    OWNER_ROLE["permissions"].append(MISSION_UPDATE_GROUPS)
    OWNER_ROLE["permissions"].append(MISSION_DELETE)
    OWNER_ROLE["permissions"].append(MISSION_SET_ROLE)
    OWNER_ROLE["permissions"].append(MISSION_READ)

    SUBSCRIBER_ROLE = {"type": MISSION_SUBSCRIBER, "permissions": []}
    SUBSCRIBER_ROLE["permissions"].append(MISSION_READ)
    SUBSCRIBER_ROLE["permissions"].append(MISSION_WRITE)

    READ_ONLY_ROLE = {"type": MISSION_READ_ONLY, "permissions": []}
    READ_ONLY_ROLE["permissions"].append(MISSION_READ)

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    clientUid: Mapped[str] = mapped_column(String(255))
    username: Mapped[str] = mapped_column(String(255))
    createTime: Mapped[datetime] = mapped_column(DateTime)
    role_type: Mapped[str] = mapped_column(String(255))
    mission_name: Mapped[str] = mapped_column(String(255), ForeignKey("missions.name"))
    mission = relationship("Mission", back_populates="roles", uselist=False)

    def serialize(self):
        return {
            "clientUid": self.clientUid,
            "username": self.username,
            "createTime": self.createTime,
            "role_type": self.role_type,
        }

    def to_json(self):
        json = {
            "clientUid": self.clientUid,
            "username": self.username,
            "createTime": iso8601_string_from_datetime(self.createTime),
            "role": {"type": self.role_type, "permissions": []},
        }

        if self.role_type == self.MISSION_OWNER:
            json["role"] = self.OWNER_ROLE

        elif self.role_type == self.MISSION_SUBSCRIBER:
            json["role"] = self.SUBSCRIBER_ROLE

        else:
            json["role"] = self.READ_ONLY_ROLE

        return json



"""Added plugins table

Revision ID: f310052349cb
Revises: c9fa07339363
Create Date: 2025-04-24 02:42:00.205913

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "f310052349cb"
down_revision = "c9fa07339363"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "plugins",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=255), nullable=False),
        sa.Column("enabled", sa.Boolean(), nullable=False),
        sa.Column("author", sa.String(length=255), nullable=True),
        sa.Column("version", sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("plugins")
    # ### end Alembic commands ###



"""Added tokens table

Revision ID: d68964dab66b
Revises: b0c97a4ce0df
Create Date: 2025-05-07 02:26:29.670335

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "d68964dab66b"
down_revision = "b0c97a4ce0df"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "tokens",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("username", sa.String(length=255), nullable=True),
        sa.Column("max_uses", sa.Integer(), nullable=True),
        sa.Column("total_uses", sa.Integer(), nullable=True),
        sa.Column("creation", sa.BigInteger(), nullable=False),
        sa.Column("not_before", sa.BigInteger(), nullable=True),
        sa.Column("expiration", sa.BigInteger(), nullable=True),
        sa.Column("disabled", sa.Boolean(), nullable=True),
        sa.Column("token_hash", sa.String(length=255), nullable=False),
        sa.ForeignKeyConstraint(
            ["username"],
            ["user.username"],
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("token_hash"),
        sa.UniqueConstraint("username"),
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    op.drop_table("tokens")
    # ### end Alembic commands ###



"""Added mission_change_id column to mission_uids table

Revision ID: 4f0173cdb93b
Revises: 00546817d518
Create Date: 2024-10-11 20:52:40.425560

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "4f0173cdb93b"
down_revision = "00546817d518"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("mission_uids", schema=None) as batch_op:
        batch_op.add_column(sa.Column("mission_change_id", sa.Integer(), nullable=True))
        batch_op.create_foreign_key(
            "mission_change", "mission_changes", ["mission_change_id"], ["id"]
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("mission_uids", schema=None) as batch_op:
        batch_op.drop_constraint(None, type_="foreignkey")
        batch_op.drop_column("mission_change_id")

    # ### end Alembic commands ###



"""Added mission_logs table

Revision ID: bde915ea136e
Revises: d31bff4a15c7
Create Date: 2024-10-10 18:13:29.597437

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "bde915ea136e"
down_revision = "d31bff4a15c7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "mission_logs",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("content", sa.String(length=255), nullable=False),
        sa.Column("creator_uid", sa.String(length=255), nullable=False),
        sa.Column("entry_uid", sa.String(length=255), nullable=False),
        sa.Column("mission_name", sa.String(length=255), nullable=False),
        sa.Column("server_time", sa.DateTime(), nullable=False),
        sa.Column("dtg", sa.DateTime(), nullable=False),
        sa.Column("created", sa.DateTime(), nullable=False),
        sa.Column("content_hash", sa.String(length=255), nullable=True),
        sa.Column("keywords", sa.JSON(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    with op.batch_alter_table("mission_content", schema=None) as batch_op:
        batch_op.alter_column("hash", existing_type=sa.String(length=255), nullable=False)
        # batch_op.drop_index('hash_index')
        batch_op.create_unique_constraint("hash_index", ["hash"])

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("mission_content", schema=None) as batch_op:
        batch_op.drop_constraint(None, type_="unique")
        batch_op.create_index("hash_index", ["hash"], unique=1)
        batch_op.alter_column("hash", existing_type=sa.String(length=255), nullable=True)

    op.drop_table("mission_logs")
    # ### end Alembic commands ###
